//+------------------------------------------------------------------+
//|                                                  Daily Pivot.mq4 |
//|                                                         value808 |
//|                                                            v.0.1 |
//+------------------------------------------------------------------+
#property copyright "value808"
#property link      ""

#property indicator_chart_window
#property indicator_buffers 7

#define NO_TRADE_ZONE 0
#define BUY_ZONE 1
#define BUY_2_ZONE 2
#define BUY_3_ZONE 3
#define SELL_ZONE -1
#define SELL_2_ZONE -2
#define SELL_3_ZONE -3

double ExtMapBuffer0[];
double ExtMapBuffer1[];
double ExtMapBuffer2[];
double ExtMapBuffer3[];
double ExtMapBuffer4[];
double ExtMapBuffer5[];
double ExtMapBuffer6[];

//---- input parameters
extern int       Offset.Hours = 0;
extern int       Offset.Minutes = 0;
extern int       Pivot.Calculation.Method = PRICE_TYPICAL; // Corresponds to the "Applied price constants" mapping (see manual for details).
extern bool      Use.As.Pivot.Calculator = false;
extern int       Buy.Sell.Zone.Pips = 20;
extern int       Target.Zone.1.Pips = 40;
extern int       Target.Zone.2.Pips = 55;
extern bool      Use.Lines = false;
extern bool      Enable.Alerts = false;
extern int       Arrow.Code = 158;
extern int       Bars.To.Count = 500;

bool active = true;
int period = 0;
double pivotPrice = EMPTY_VALUE, buyPrice = EMPTY_VALUE, sellPrice = EMPTY_VALUE,
       buyTarget1Price = EMPTY_VALUE, buyTarget2Price = EMPTY_VALUE,
       sellTarget1Price = EMPTY_VALUE, sellTarget2Price = EMPTY_VALUE;

int lastAlertZone = NO_TRADE_ZONE;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
   if (Period()>PERIOD_H4) {
      Alert("Not suitable for periods higher than H4!");
      active = false;
   }
   
   int indicatorType = 0;
   if (Use.Lines) indicatorType = DRAW_LINE; else indicatorType = DRAW_ARROW;
   
   SetIndexStyle(0,indicatorType,EMPTY,1,C'0x10,0xFF,0x10');
   SetIndexArrow(0,Arrow.Code);
   SetIndexBuffer(0,ExtMapBuffer0);
   SetIndexStyle(1,indicatorType,EMPTY,1,C'0x10,0x10,0xFF');
   SetIndexArrow(1,Arrow.Code);
   SetIndexBuffer(1,ExtMapBuffer1);
   SetIndexStyle(2,indicatorType,EMPTY,1,C'0xFF,0x10,0x10');
   SetIndexArrow(2,Arrow.Code);
   SetIndexBuffer(2,ExtMapBuffer2);
   SetIndexStyle(3,indicatorType,EMPTY,1,C'0x10,0x70,0xFF');
   SetIndexArrow(3,Arrow.Code);
   SetIndexBuffer(3,ExtMapBuffer3);
   SetIndexStyle(4,indicatorType,EMPTY,1,C'0xFF,0x70,0x10');
   SetIndexArrow(4,Arrow.Code);
   SetIndexBuffer(4,ExtMapBuffer4);
   SetIndexStyle(5,indicatorType,EMPTY,1,C'0x70,0x70,0xFF');
   SetIndexArrow(5,Arrow.Code);
   SetIndexBuffer(5,ExtMapBuffer5);
   SetIndexStyle(6,indicatorType,EMPTY,1,C'0xFF,0x70,0x70');
   SetIndexArrow(6,Arrow.Code);
   SetIndexBuffer(6,ExtMapBuffer6);
   
   period = PERIOD_D1/Period();
      
   return(0);
  }
  
void displayMessages(int offset) {

   double currentPrice = Close[offset];
	  
   if (offset == 0) {
	  if ((currentPrice >= pivotPrice && lastAlertZone == SELL_ZONE) ||
	      (currentPrice <= pivotPrice && lastAlertZone == BUY_ZONE)) {
	     if (Enable.Alerts) Alert(Symbol()+": Pivot reached. Please close open positions.");
	     Print (Symbol()+": Pivot reached. Please close open positions.");
	     lastAlertZone=NO_TRADE_ZONE;
	  }
	  if (currentPrice>=buyPrice && lastAlertZone == NO_TRADE_ZONE) {
	     if (Enable.Alerts) Alert(Symbol()+": Buy zone reached.");
	     Print (Symbol()+": Buy zone reached.");
	     lastAlertZone=BUY_ZONE;
	  }
	  if (currentPrice>=buyTarget1Price && lastAlertZone == BUY_ZONE) {
	     if (Enable.Alerts) Alert(Symbol()+": 1st buy target zone reached.");
	     Print (Symbol()+": 1st buy target zone reached.");
	     lastAlertZone=BUY_2_ZONE;
	  }
	  if (currentPrice>=buyTarget2Price && lastAlertZone == BUY_2_ZONE) {
	     if (Enable.Alerts) Alert(Symbol()+": 2nd buy target zone reached.");
	     Print (Symbol()+": 2nd buy target zone reached.");
	     lastAlertZone=BUY_3_ZONE;
	  }
	  if (currentPrice<=sellPrice && lastAlertZone == NO_TRADE_ZONE) {
	     if (Enable.Alerts) Alert(Symbol()+": Sell zone reached.");
	     Print (Symbol()+": Sell zone reached.");
	     lastAlertZone=SELL_ZONE;
	  }
	  if (currentPrice<=sellTarget1Price && lastAlertZone == SELL_ZONE) {
	     if (Enable.Alerts) Alert(Symbol()+": 1st sell target zone reached.");
	     Print (Symbol()+": 1st sell target zone reached.");
	     lastAlertZone=SELL_2_ZONE;
	  }
	  if (currentPrice<=sellTarget2Price && lastAlertZone == SELL_2_ZONE) {
	     if (Enable.Alerts) Alert(Symbol()+": 2nd sell target zone reached.");
	     Print (Symbol()+": 2nd sell target zone reached.");
	     lastAlertZone=SELL_3_ZONE;
	  }
   }
}
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
  {
   if (!active) return (1);
	int counted_bars = IndicatorCounted();
	if (counted_bars<0)  { return (-1); }

   int limit = Bars - counted_bars - 1;	

   if (Bars.To.Count != 0 && limit > Bars.To.Count) limit = Bars.To.Count;
	
	int lastDay = 0;
	
	for (int i=limit; i>=0; i--) {
	  
	  datetime timeBar = iTime(NULL,0,i);
	  
	  int curDay = TimeDayOfWeek(timeBar);
	  int curHour = TimeHour(timeBar);
	  int curMinute = TimeMinute(timeBar);
	  
	  if (lastDay != curDay && ((curHour == Offset.Hours && curMinute >= Offset.Minutes) ||
	        (curHour > Offset.Hours))) {
	  
	  	  lastDay = curDay;
   	  
   	  datetime pivotChangeTime = timeBar;

 	     if (curHour != Offset.Hours) {
 	       pivotChangeTime -= curHour*3600;
  	       pivotChangeTime += Offset.Hours*3600;
  	     }

        if (curMinute != Offset.Minutes) {
      	  pivotChangeTime -= curMinute*60;
      	  pivotChangeTime += Offset.Minutes*60;
        }

    	  datetime openTime = 0;
    	  if (curDay == 1) {
    	    openTime = pivotChangeTime-3*PERIOD_D1*60; 
    	  } else {
    	    openTime = pivotChangeTime-PERIOD_D1*60;
    	  }
 	     
	     int barShiftOpen = iBarShift(NULL,PERIOD_M1,openTime,false);
	     int barShiftClose = iBarShift(NULL,PERIOD_M1,pivotChangeTime-PERIOD_M1*60,false);
 	     int periods = barShiftOpen - barShiftClose;

	     double openPrice = iOpen(NULL,PERIOD_M1,barShiftOpen);
	     double closePrice = iClose(NULL,PERIOD_M1,barShiftClose);
	     double highPrice = iHigh(NULL,PERIOD_M1,Highest(NULL,PERIOD_M1,MODE_HIGH,periods,barShiftClose));
	     double lowPrice = iLow(NULL,PERIOD_M1,Lowest(NULL,PERIOD_M1,MODE_LOW,periods,barShiftClose));

        switch (Pivot.Calculation.Method) {
            case PRICE_CLOSE:    pivotPrice = closePrice; break;
            case PRICE_OPEN:     pivotPrice = openPrice; break;
            case PRICE_HIGH:     pivotPrice = highPrice; break;
            case PRICE_LOW:      pivotPrice = lowPrice; break;
            case PRICE_MEDIAN:   pivotPrice = (highPrice+lowPrice)/2; break;
            case PRICE_TYPICAL:  pivotPrice = (highPrice+lowPrice+closePrice)/3; break;
            case PRICE_WEIGHTED: pivotPrice = (highPrice+lowPrice+2*closePrice)/4; break;
        }
	     
	     if (!Use.As.Pivot.Calculator) {
	        buyPrice = pivotPrice + Buy.Sell.Zone.Pips*Point;
	        sellPrice = pivotPrice - Buy.Sell.Zone.Pips*Point;
	        buyTarget1Price = pivotPrice + Target.Zone.1.Pips*Point;
	        sellTarget1Price = pivotPrice - Target.Zone.1.Pips*Point;
	        buyTarget2Price = pivotPrice + Target.Zone.2.Pips*Point;
	        sellTarget2Price = pivotPrice - Target.Zone.2.Pips*Point;
	     }
	  
	  }
      
	  ExtMapBuffer0[i]=pivotPrice;

     if (!Use.As.Pivot.Calculator) {
	     ExtMapBuffer1[i]=buyPrice;
	     ExtMapBuffer2[i]=sellPrice;
	     ExtMapBuffer3[i]=buyTarget1Price;
	     ExtMapBuffer4[i]=sellTarget1Price;
	     ExtMapBuffer5[i]=buyTarget2Price;
	     ExtMapBuffer6[i]=sellTarget2Price;
	  
        displayMessages(i);
     }
     	  
	}

   return(0);
  }
//+------------------------------------------------------------------+